package com.biganto.visual.roompark.presentation.screen.deal

import android.content.Context
import androidx.annotation.StringRes
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.DealInteractor
import com.biganto.visual.roompark.domain.model.*
import com.biganto.visual.roompark.presentation.screen.deal.DealScreen
import com.biganto.visual.roompark.presentation.screen.estate.DealScreen
import com.biganto.visual.roompark.presentation.screen.estate.DealScreenViewState
import com.biganto.visual.roompark.presentation.screen.estate.InfoShowType
import com.biganto.visual.roompark.presentation.screen.estate.util.DisplayInfoModel
import com.biganto.visual.roompark.util.extensions.toRubly
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class DealScreenPresenter @Inject constructor(
    private val interactor: DealInteractor,
    private val context: Context,
    @Named(SELECTED_ESTATE_ID_KEY) private val estateId:Int
)
    : BigantoBasePresenter<DealScreen, DealScreenViewState>() {

    private var planList: List<PlanPresetModel>? = null

    private var estate: DealModel? = null

    private var showType: InfoShowType = InfoShowType.COMMON_INFO

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> DealScreenViewState.SomeError(e) }

    private fun getPlan(plan: PlanPresetModel): Observable<DealScreenViewState> =
        interactor.getPlan(plan)
            .map<DealScreenViewState> { DealScreenViewState.LoadPlan(it) }

    override fun bindIntents() {

        val prefetchCards = interactor.getDeal(estateId)
            .doOnNext { estate = it.copy() }
            .map { DealScreenViewState.LoadDeal(it) }

        val state = restoreStateObservable
            .mergeWith(prefetchCards)
            .doOnError { Timber.e(it) }
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(DealScreenViewState::class.java), DealScreen::render)
    }

    private fun langString(@StringRes id:Int) = context.resources.getString(id)

    private fun langString(@StringRes id:Int,vararg args:Any) =
            context.resources.getString(id,*args)

}

